@@ -2,6 +2,7 @@
 
 from __future__ import annotations
 
+import json
 import os
 import subprocess
 import sys
@@ -386,13 +387,6 @@ def manual_step(
         ),
     ],
     cmd: Annotated[str, typer.Option("--cmd", help="Command to run.")] = None,
-    shell: Annotated[
-        bool,
-        typer.Option(
-            "--shell",
-            help="Whether or not to execute the command in shell mode.",
-        ),
-    ] = False,
     show_stdout: Annotated[
         bool, typer.Option("--show-stdout", help="Show stdout.")
     ] = False,
@@ -403,17 +397,17 @@ def manual_step(
     if cmd is not None:
         typer.echo(f"Running command: {cmd}")
         subprocess.Popen(
-            cmd.split() if not shell else cmd,
+            cmd,
             stderr=subprocess.PIPE if not show_stderr else None,
             stdout=subprocess.PIPE if not show_stdout else None,
-            shell=shell,
+            shell=True,
         )
     input(message + " (press enter to confirm): ")
     typer.echo("Done")
 
 
 @app.command(
-    name="run-env",
+    name="runenv",
     help="Run a command in an environment.",
     context_settings={"ignore_unknown_options": True},
 )
@@ -485,3 +479,81 @@ def run_in_env(
     else:
         typer.echo("Environment kind not supported", err=True)
         raise typer.Exit(1)
+
+
+@app.command(
+    name="check-call",
+    help=(
+        "Check that a call to a command succeeds and run another command "
+        "if there is an error."
+    ),
+)
+def check_call(
+    cmd: Annotated[str, typer.Argument(help="Command to check.")],
+    if_error: Annotated[
+        str,
+        typer.Option(
+            "--if-error", help="Command to run if there is an error."
+        ),
+    ],
+):
+    try:
+        subprocess.check_call(cmd, shell=True)
+        typer.echo("Command succeeded")
+    except subprocess.CalledProcessError:
+        typer.echo("Command failed")
+        try:
+            typer.echo("Attempting fallback call")
+            subprocess.check_call(if_error, shell=True)
+            typer.echo("Fallback call succeeded")
+        except subprocess.CalledProcessError:
+            typer.echo("Fallback call failed", err=True)
+            raise typer.Exit(1)
+
+
+@app.command(
+    name="build-docker",
+    help="Build Docker image if missing or different from lock file.",
+)
+def build_docker(
+    tag: Annotated[str, typer.Argument(help="Image tag.")],
+    fpath: Annotated[
+        str, typer.Option("-i", "--input", help="Path to input Dockerfile.")
+    ] = "Dockerfile",
+):
+    def get_docker_inspect():
+        out = json.loads(
+            subprocess.check_output(["docker", "inspect", tag]).decode()
+        )
+        # Remove some keys that can change without the important aspects of
+        # the image changing
+        _ = out[0].pop("Id")
+        _ = out[0].pop("RepoDigests")
+        _ = out[0].pop("Metadata")
+        return out
+
+    typer.echo(f"Checking for existing image with tag {tag}")
+    # First call Docker inspect
+    try:
+        inspect = get_docker_inspect()
+    except subprocess.CalledProcessError:
+        typer.echo(f"No image with tag {tag} found locally")
+        inspect = []
+    lock_fpath = fpath + "-lock.json"
+    rebuild = True
+    if os.path.isfile(lock_fpath):
+        typer.echo(f"Reading lock file: {lock_fpath}")
+        with open(lock_fpath) as f:
+            lock = json.load(f)
+    else:
+        typer.echo(f"Lock file ({lock_fpath}) does not exist")
+        lock = None
+    if inspect and lock:
+        typer.echo("Checking image against lock file")
+        rebuild = inspect[0]["RootFS"]["Layers"] != lock[0]["RootFS"]["Layers"]
+    if rebuild:
+        subprocess.check_call(["docker", "build", "-t", tag, "-f", fpath, "."])
+    # Write the lock file
+    inspect = get_docker_inspect()
+    with open(lock_fpath, "w") as f:
+        json.dump(inspect, f, indent=4)
